# Testing

## Unit Tests
### Local Setup
For downloading tools for testing locally, first run following command
```bash
bash ./Tests/setup-tests-env.sh
```

### Execution
Unit tests are based on linux executables so if you are running on a windows machine you should switch to a WSL(Windows Subsystem for Linux) terminal. Running the whole package of unit tests can simply be done by running the following command from the root of the git repository: 
```bash
# <path to the bats executable> <path to the test folder>
./Tests/Tools/bats-core/bin/bats ./Tests/Unit_Tests/ -r
```

Optionally:
- If you are using VS Code you can use the launch config with the name: `Run all unit test`. The only caveat is that you need to be in the workspace directory folder. 
- You can also run `./Tests/run-all-tests.bat` from the windows command line.

### Creating tests
To create a test you will simply need to create a file (if it does not exist) with the `.bats` extension. The name of the `test file` should be the same as the name of the `tested file` and should be placed in the correct folder (see `Directory Structure` section below). 

If you want the test file to be an executable itself, you can add (make sure that the `bats` path is correct):
```bash
#!/usr/bin/env ../Tools/bats-core/bin/bats
```
The next section should be where you load `helpers`. It can be done like this:
```bash
load '../Tools/bats-support/load'
load '../Tools/bats-assert/load'
load '../Tools/bats-mock/load'
```
In the same manner you can load the file that you want to test.

In each test file you can have two predefined functions that will be run at the begining and end of `each` test:
```bash
setup()
teardown()
```
Using these functions you can add and remove mocking and any other prerequsites your tests might need. You can also do that in the test itself. See `example.bats`.

All output is captured by `BATS`. For debugging purposes you might want to use something like:
```bash
echo "# DEBUG $output" >&3
```

Here is an example of a basic test:

```bash
@test "EXAMPLE: check exit code and message on 'error' function" {
  # Arrange
  # Act
  run error "TEST"
  
  # Assert
  assert_failure
  assert_output --partial "TEST"
}
```
Please keep the Arrange / Act / Assert structure inside the test. 
#### Useful to know:
- `stub` / `unstub` function that will allow you to mock. See `example.bats`:
```bash
stub kubectl \
  "get pods : echo 'foo'"
```
Where we are mocking kubectl's output for `get pods` to echo out `foo`.
- mocking can also be done like as you can see in the below example. We are mocking the `jq` cmdlet. 
```bash
function jq {
      echo test_complete
  }
export -f jq
# Use unset to unmock
unset jq
```
- `run` function allows you to run the tested function
- `assert_success` and `assert_failure` -> if the command exited with 0 or not
- `asssert_output` it checks the `$output` from the run command against a string. It has a `--partial` flag.
- you can also assert like this:
```bash
[ "$output" = "test_complete" ]
```

Additional docs (but not that complex) can be found:
- [here](https://github.com/bats-core/bats-core/blob/master/docs/source/writing-tests.md) for general `BATS`
- [here](https://github.com/jasonkarns/bats-mock) for `bats-mock`
- [here](https://github.com/ztombol/bats-assert) for `bats-assert`
- [here](https://github.com/ztombol/bats-support) for `bats-support`

### Directory structure
The root folder `Tests` contains:
- `Tools` folder that will have the tools needed for testing. See the `Details` section. These are submodules to github repositories.
- `Unit_Tests` folder that will contain all unit tests. This was chosen as a separate folder so it won't be harder when you package other folders.
Since the `tests file` will generally be larger than the `tested file` itself, we should replicate the folder structure from the project but only one level deep to keep things simple.

### Using submodules
To update the submodules you can use the following command. This will create changes to modules that will need to be checked in and commited as you would normally do for code.
```cmd
git submodule update --remote
```

### Details
We are using [BATS](https://github.com/bats-core/bats-core) for unit testing with a few modules:
- [bats-assert](https://github.com/ztombol/bats-assert)
- [bats-support](https://github.com/ztombol/bats-support)
- [bats-mock](https://github.com/jasonkarns/bats-mock)