function testRancherServerPodCount() {
    local expectedRancherServerPodCount expectedWebhookPodCount replicaCount podCount validTestRailIds requestedTestRailIds
    expectedRancherServerPodCount=$1
    expectedWebhookPodCount=$2
    requestedTestRailIds=$3

    validTestRailIds=("1621320" "1621321" "1663794" "1663802")
    for validTestRailId in "${validTestRailIds[@]}"; do
        if [[ "${requestedTestRailIds}" =~ "${validTestRailId}" ]]; then
            result="false";
            replicaCount=$(fetchReplicaCount deploy cattle-system rancher-server)
            if [[ -z $replicaCount && -z $expectedRancherServerPodCount && -z $expectedWebhookPodCount ]]; then
                result="true"
                appendOutput "${validTestRailId}" "${result}"
                return
            fi
            podCount=$(kubectl -n cattle-system get pods --field-selector=status.phase=Running --selector=app=rancher-server --output json | jq -j ".items | length")
            if [[ ${podCount} -ge ${expectedRancherServerPodCount} && ${podCount} -le ${replicaCount} ]]; then
                result="true"
            fi

            replicaCount=$(fetchReplicaCount deploy cattle-system rancher-webhook)
            podCount=$(kubectl -n cattle-system get pods --field-selector=status.phase=Running --selector=app=rancher-webhook --output json | jq -j ".items | length")
            if [[ ${podCount} -ge ${expectedWebhookPodCount} && ${podCount} -le ${replicaCount} && "${result}" == "true" ]]; then
                result="true"
            fi

            appendOutput "${validTestRailId}" "${result}"
        fi
    done
}

function setRancherServerValidationParam() {
    local validationStage
    validationStage=$1
    case "$validationStage" in
    3nodeup)
        rancherServerPod=2
        webhookPod=1
        testRailIds="1621320"
        ;;
    rebootAllServers)
        rancherServerPod=2
        webhookPod=1
        testRailIds="1621321"
        ;;
    2nodeup)
        rancherServerPod=2
        webhookPod=1
        testRailIds="1663794"
        ;;
    1nodeup|0nodeup)
        rancherServerPod=""
        webhookPod=""
        testRailIds="1663802"
        ;;
    *)
        echo "Unexpected option ${validationStage} passed to ${FUNCNAME[0]}"
        exit 1
        ;;
    esac

}

function executeRancherServerTest() {
    setRancherServerValidationParam "${VALIDATE_STAGE}"
    testRancherServerPodCount "${rancherServerPod}" "${webhookPod}" "${testRailIds}"
}