#!/bin/bash

if ! test -d /opt/UiPathAutomationSuite/UiPath_Installer ; then
  echo "Dir: /opt/UiPathAutomationSuite/UiPath_Installer does not exist"
fi
cd /opt/UiPathAutomationSuite/UiPath_Installer || exit 1

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}/utils.sh"

export KUBECONFIG=/etc/rancher/rke2/rke2.yaml PATH=$PATH:/var/lib/rancher/rke2/bin

function fill_dummy_data_in_pod_mount_point() {
  local mount_point="$1"
  local total_bytes="$2"
  local pod_name="$3"
  for i in {1..3}; do 
    #shellcheck disable=SC2015
    kubectl exec "${pod_name}" -- sh -c "head -c ${total_bytes} </dev/urandom >${mount_point}" && echo "Wrote dummy data to pod" && break || sleep 30; 
  done
}

function test_KubePersistentVolumeFillingUpCritical() {
  kubectl apply -f ~/resources/pvc-with-pod.yaml || true
  
  fill_dummy_data_in_pod_mount_point "/data/myfile" "94857600" "test-pod"
  #shellcheck disable=SC2015
  [[ "$(is_alert_fired "KubePersistentVolumeFillingUp" "test-pvc" ".labels.persistentvolumeclaim")" == "true" ]] && appendOutput "1658508" "true" || appendOutput "1658508" "false"
  #shellcheck disable=SC2015
  [[ "$(is_alert_fired "KubePersistentVolumeFillingUpWarning" "test-pvc" ".labels.persistentvolumeclaim")" == "true" ]] && appendOutput "1658509" "true" || appendOutput "1658509" "false"
  
  kubectl delete -f ~/resources/pvc-with-pod.yaml
}

