#!/bin/bash

if ! test -d /opt/UiPathAutomationSuite/UiPath_Installer ; then
  echo "Dir: /opt/UiPathAutomationSuite/UiPath_Installer does not exist"
fi
cd /opt/UiPathAutomationSuite/UiPath_Installer || exit 1

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}/utils.sh"



function export_s3_config() {
  export KUBECONFIG=/etc/rancher/rke2/rke2.yaml PATH=$PATH:/var/lib/rancher/rke2/bin
  OBJECT_GATEWAY_INTERNAL_IP="rook-ceph-rgw-rook-ceph"
  OBJECT_GATEWAY_INTERNAL_HOST=$(kubectl -n rook-ceph get services/$OBJECT_GATEWAY_INTERNAL_IP -o jsonpath="{.spec.clusterIP}")
  OBJECT_GATEWAY_INTERNAL_PORT=$(kubectl -n rook-ceph get services/$OBJECT_GATEWAY_INTERNAL_IP -o jsonpath="{.spec.ports[0].port}")
  STORAGE_ACCESS_KEY=$(kubectl -n rook-ceph get secret ceph-object-store-secret -o json | jq '.data.OBJECT_STORAGE_ACCESSKEY' | sed -e 's/^"//' -e 's/"$//' | base64 -d)
  STORAGE_SECRET_KEY=$(kubectl -n rook-ceph get secret ceph-object-store-secret -o json | jq '.data.OBJECT_STORAGE_SECRETKEY' | sed -e 's/^"//' -e 's/"$//' | base64 -d)
  export AWS_HOST=$OBJECT_GATEWAY_INTERNAL_HOST
  export AWS_ENDPOINT=$OBJECT_GATEWAY_INTERNAL_HOST:$OBJECT_GATEWAY_INTERNAL_PORT
  export AWS_ACCESS_KEY_ID=$STORAGE_ACCESS_KEY
  export AWS_SECRET_ACCESS_KEY=$STORAGE_SECRET_KEY
}

function test_ceph_alerts() {

  export_s3_config
  #create bucket
  /usr/local/bin/s3cmd mb --host="${AWS_HOST}" --host-bucket=  s3://hello-bucket --no-ssl
  #Fill disk 75% initially
  dd if=/dev/zero of=/mnt/random.txt count=7700 bs=1048576
  /usr/local/bin/s3cmd put /mnt/random.txt --no-ssl --host="${AWS_HOST}" --host-bucket=  s3://hello-bucket/random.txt
  #shellcheck disable=SC2015
  [[ "$(is_alert_fired "CephClusterNearFull" "rook-ceph-mgr" ".labels.job")" == "true" ]] && appendOutput "1664046" "true" || appendOutput "1664046" "false"
  #shellcheck disable=SC2015
  [[ "$(is_alert_fired "CephOSDNearFull" "rook-ceph-mgr" ".labels.job")" == "true" ]] && appendOutput "1664046" "true" || appendOutput "1664046" "false"
  #Fill up to 85% now
  dd if=/dev/zero of=/mnt/random.txt count=1024 bs=1048576
  /usr/local/bin/s3cmd put /mnt/random.txt --no-ssl --host="${AWS_HOST}" --host-bucket=  s3://hello-bucket/random1.txt
  #shellcheck disable=SC2015
  [[ "$(is_alert_fired "CephClusterCriticallyFull" "rook-ceph-mgr" ".labels.job")" == "true" ]] && appendOutput "1664047" "true" || appendOutput "1664047" "false"
  #shellcheck disable=SC2015
  [[ "$(is_alert_fired "CephClusterReadOnly" "rook-ceph-mgr" ".labels.job")" == "true" ]] && appendOutput "1664048" "true" || appendOutput "1664048" "false"
  #shellcheck disable=SC2015
  [[ "$(is_alert_fired "CephOSDCriticallyFull" "rook-ceph-mgr" ".labels.job")" == "true" ]] && appendOutput "1664047" "true" || appendOutput "1664047" "false"
  rm -f /mnt/random.txt &>/dev/null || true
}

