#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

set -eu

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
if [[ -z "${SCRIPT_DIR}" ]]; then
  error "Could not determine script path"
fi
MODULES_DIR=$(cd "${SCRIPT_DIR}/../../Modules" && pwd)
if [[ -z "${MODULES_DIR}" ]]; then
  error "Could not determine modules path"
fi

readonly SCRIPT_DIR
readonly MODULES_DIR

#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/upgrade_utils.sh"
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}"/longhornutils.sh
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}"/k8sutils.sh
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}"/utils.sh

function parse_input_file() {
  local profile
  profile="$(get_mandatory_json_value ".profile")"
  if [[ "${profile}" == "ha" ]];
  then
    INSTALL_MULTINODE="true"
  else
    INSTALL_MULTINODE="false"
  fi
  export INSTALL_MULTINODE
}

function recreate_storage_class() {
    local storageClassName=$1
    local expectedReplicaCount=$2
    local storageClassFileName=$3
    local storageClassTmpFileName=$4
    local current_replica_count
    local storage_class_name
    storage_class_name=$(kubectl get storageclass -o json | jq -r ".items[]  | select(.metadata.name == \"${storageClassName}\") | .spec.volumeName")
    if [ -n "$storage_class_name" ]
    then
        current_replica_count=$(kubectl -n longhorn-system get storageclass "${storageClassName}" -o json | jq -r '.parameters.numberOfReplicas')
        if [[ "$current_replica_count" != "${expectedReplicaCount}" || "$current_replica_count" == "" ]]
        then
            kubectl delete storageclass "${storageClassName}" &>/dev/null
            rm -f "$SCRIPT_DIR"/../../Infra_Installer/longhorn/"${storageClassTmpFileName}" &>/dev/null
            cp "$SCRIPT_DIR"/../../Infra_Installer/longhorn/"${storageClassFileName}" "$SCRIPT_DIR"/../../Infra_Installer/longhorn/"${storageClassTmpFileName}"
            sed -i "s/REPLICA_PLACEHOLDER/${expectedReplicaCount}/" "$SCRIPT_DIR"/../../Infra_Installer/longhorn/"${storageClassTmpFileName}"
            kubectl apply -f "$SCRIPT_DIR"/../../Infra_Installer/longhorn/"${storageClassTmpFileName}"  1>/dev/null
        fi 
    fi
}

# $1=longhorn-backup-single-replica
function update_replica_count() {
    local storageClassName=$1
    local replicaCount=$2
    set_longhorn_url
    local volumes
    volumes=$(kubectl get pvc -A -o json | jq -r ".items[]  | select(.spec.storageClassName ==\"${storageClassName}\") | .spec.volumeName")
    for volume in $volumes; do
        kubectl -n longhorn-system patch volumes.longhorn.io "${volume}" --type='json' -p "[{ \"op\": \"replace\", \"path\": \"/spec/numberOfReplicas\", \"value\": ${replicaCount} }]" 1>/dev/null
    done
}

if [[ "${NODE_ROLE}" == "none" ]];
then
  parse_input_file
  if [[ "${INSTALL_MULTINODE}" == "true" ]]; then
    recreate_storage_class "longhorn-backup-single-replica" "2" "longhorn-storage-backup-one-replica.yaml" "longhorn-storage-backup-one-replica-tmp.yaml"
    update_replica_count "longhorn-backup-single-replica" "2"
    recreate_storage_class "longhorn-backup" "2" "longhorn-storage-backup.yaml" "longhorn-storage-backup-tmp.yaml"
    update_replica_count "longhorn-backup" "2"
    recreate_storage_class "longhorn" "1" "longhorn-storage.yaml" "longhorn-storage-tmp.yaml"
    update_replica_count "longhorn" "1"
  fi
fi

