#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

#GLOBALS:
SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
if [[ -z "${SCRIPT_DIR}" ]]; then
  error "Could not determine script path"
fi
MODULES_DIR=$(cd "${SCRIPT_DIR}/../../Modules" && pwd)
if [[ -z "${MODULES_DIR}" ]]; then
  error "Could not determine modules path"
fi
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/utils.sh"
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/upgrade_utils.sh"
#shellcheck disable=SC1090,SC109
source "${MODULES_DIR}/argocd_utils.sh"



function check_ceph_health() {
  local rook_cluster_namespace=rook-ceph
  current_rook_version=$(kubectl -n $rook_cluster_namespace get deployment -l rook_cluster=$rook_cluster_namespace -o jsonpath='{range .items[*]}{"rook-version="}{.metadata.labels.rook-version}{"\n"}{end}' | sort | uniq)
  # current_ceph_version=$(kubectl -n $rook_cluster_namespace get deployment -l rook_cluster=$rook_cluster_namespace -o jsonpath='{range .items[*]}{"ceph-version="}{.metadata.labels.ceph-version}{"\n"}{end}' | sort | uniq)
  if [[ "${current_rook_version}" == "rook-version=v1.5.9" ]];
  then
      # We are still on older version and we will be upgrading to rook v1.8.9-4.gb6f35a55f, hence validate ceph health first
      ceph_health_exit_on_error
      
  fi
}

function main() {
  if [[ "$(is_rook_ceph_installed)" == "false" ]]; then
    warn "Skipping rook-ceph-pre-1.8.9.sh because rook-ceph is not installed"
    return
  fi
  
  check_ceph_health
  handle_device_health_metrics_pool_config
  speed_up_ceph_recovery || true
  wait_until_ceph_pgs_active || error "Timeout waiting for all PGs to become active+clean before upgrade"
}

main
exit 0